; from Anthony Hill posted in cosmacelf groups.io Dec 13 2023
;
;  1802 MEMORY TESTER
;
;  - starts at memory page input from from front panel switches via an INP4
;  - outputs # of passes to from panel LED's via OUT4
;  - blinks Q on each pass through memory
;  - stops on error, turns on Q, and outputs memory page # on front panel LED's
;  - amount of memory to test is determined by setting byte_count
;


R2 EQU $02              ; R2 = stack pointer
R4 EQU $04              ; R4 = test pass count
R9 EQU $09              ; R9 = pattern data byte to write in memory
RA EQU $0A              ; RA -> address to start testing memory
RB EQU $0B              ; RB = number of bytes to test
RC EQU $0C              ; RC -> memory address being test
RD EQU $0D              ; RD = byte pattern counter
RE EQU $0E              ; RE = value to save in memory (increments to 251 then resets to zero)

byte_count EQU $2000                ; # of memory bytes to test on each pass

    ORG $0000                       ; default run address
    DIS                             ; disable interrupts
    DB $00                          ;
    REQ                             ; turn off Q  (the error indicator)
                                    ;
START:                              ;
    LDI high STACK                  ;  R2 = stack pointer
    PHI R2                          ;
    LDI low STACK                   ;
    PLO R2                          ;
    SEX R2                          ;
                                    ;
    LDI $FF                         ; reset the LED front panel display
    STR R2                          ;
    OUT 4                           ;
    DEC R2                          ;
                                    ;
    LDI $00                         ; set R4 (the test pass counter) to zero
    PHI R4                          ;
    PLO R4                          ;
                                    ;
    INP  4                          ; get high byte start of test address from front panel switches
    LSNZ                            ;
    LDI $01                         ; don't allow it to start on zero page
    PHI RA                          ; set RA = start of memory test
    LDI $00                         ;
    PLO RA                          ;
                                    ;
    LDI high byte_count             ;  RB = number of bytes to test
    PHI RB                          ;
    LDI low byte_count              ;
    PLO RB                          ;
                                    ;
    LDI $00                         ;  R9 = count value to write into memory
    PLO R9                          ;
                                    ;
MAIN_LOOP:                          ; loop back here after each set & test pass through memory
    GHI RA                          ; reload pointers
    PHI RC                          ; reset RC to start of memory test address
    GLO RA                          ;
    PLO RC                          ;
    GHI RB                          ; reset RD to number of bytes to test
    PHI RD                          ;
    GLO RB                          ;
    PLO RD                          ;
    GLO R9                          ; RE.0 = prime reset counter first byte to use
    PLO RE
;
;  fill memory
;
FILL:
    GHI RC                          ; don't let memory test wrap around and corrupt code on page zero
    BNZ FL1                         ;
    GLO RC                          ;
    BZ  TEST                        ;
FL1:                                ;
    GLO RE                          ;  fill memory with prime reset counter
    STR RC                          ;
    INC RC                          ; get next value for prime reset counter into RE.0 and
                                    ;
    INC RE                          ; reset RE.0 at 251 (larges prime number less than 256 )
    GLO RE                          ;
    SMI 251                         ; 251 = larges prime number less than 256
    BNZ FL2                         ;
    LDI $00                         ;
    PLO RE                          ;
FL2:                                ;
    DEC RD                          ;
    GLO RD                          ;
    BNZ FILL                        ;
    GHI RD                          ;
    BNZ FILL                        ;

    LSNQ                            ; toggle Q so that we have some front panel activity (optional)
    REQ                             ;
    SKP                             ;
    SEQ                             ;

;  test filled memory

TEST:
    GHI RA                          ; reload pointers  RC=memory address being tested    RD=byte counter
    PHI RC                          ;
    GLO RA                          ;
    PLO RC                          ;
    GHI RB                          ;
    PHI RD                          ;
    GLO RB                          ;
    PLO RD                          ;
    GLO R9                          ; RE.0 = prime reset counter
    PLO RE                          ;
                                    ;
TESTLOOP:                           ;
    GHI RC                          ; don't let memory test wrap around and corrupt code on page zero
    BNZ TL1                         ;
    GLO RC                          ;
    BZ  DONE                        ;
TL1:                                ;
    LDN RC                          ;
    STR R2                          ;
    GLO RE                          ;
    SM                              ;
    BNZ error                       ;
    INC RC                          ;
                                    ;
    INC RE                          ; reset RE.0 at 251 (larges prime number less than 256 )
    GLO RE                          ;
    SMI 251                         ; 251 = larges prime number less than 256
    BNZ TL2                         ;
    LDI $00                         ;
    PLO RE                          ;
TL2:                                ;
    DEC RD                          ;
    GLO RD                          ;
    BNZ TESTLOOP                    ;
    GHI RD                          ;
    BNZ TESTLOOP                    ;

DONE:
    INC R9                          ; start the next pass with the next higher number
                                    ;
    INC R4                          ; update the count of the number of passes so far
    GLO R4                          ;
    STR R2                          ;
    OUT 4                           ; output the pass counter to the LED display
    DEC R2                          ;

    LSNQ                            ; toggle Q so that we have some front panel activity (optional)
    REQ                             ;
    SKP                             ;
    SEQ                             ;
                                    ;
    BR MAIN_LOOP                    ;

error:
    DEC R2                          ; output the high byte of the address where the error occurred to the front panel LED's
    GHI RC                          ;
    STR R2                          ;
    OUT 4                           ;
    SEQ                             ; turn on Q as well
hang:                               ;
    BR hang                         ; loop forever
                                    ;
STACK  EQU    ( $ + $0100  AND $FF00 )  - $0010

    END




